import axios from 'axios';

// import { ElLoading, ElMessage } from 'element-plus';
import qs from 'qs';

const DEAFULT_LOADING = false;

class AiotRequest {
  instance;

  interceptors;

  showLoading;

  loading;

  constructor(config) {
    // 创建axios实例
    this.instance = axios.create(config);

    // 保存基本信息
    this.showLoading = config.showLoading ?? DEAFULT_LOADING;
    this.interceptors = config.interceptors;

    // 使用拦截器
    // 1.从config中取出的拦截器是对应的实例的拦截器
    this.instance.interceptors.request.use(
      this.interceptors?.requestInterceptor,
      this.interceptors?.requestInterceptorCatch
    );
    this.instance.interceptors.response.use(
      this.interceptors?.responseInterceptor,
      this.interceptors?.responseInterceptorCatch
    );

    // 2.添加所有的实例都有的拦截器
    this.instance.interceptors.request.use(
      (config) => {
        if (this.showLoading) {
          // this.loading = ElLoading.service({
          //   lock: true,
          //   text: '正在请求数据....',
          //   background: 'rgba(0, 0, 0, 0.5)'
          // });
        }
        return config;
      },
      (err) => {
        return Promise.reject(err);
      }
    );

    this.instance.interceptors.response.use(
      (res) => {
        // 将loading移除
        this.loading?.close();
        // console.log(res)
        const { data } = res;
        if (!data) {
          console.log('请求失败~, 错误信息');
          // ElMessage({
          //   type: 'error',
          //   message: data.resultMsg
          // });
          return Promise.reject(data.resultMsg);
        }
        return data;
      },
      (err) => {
        console.log(err);
        // 将loading移除
        this.loading?.close();

        // 例子: 判断不同的HttpErrorCode显示不同的错误信息
        if (err.response && err.response.status === 404) {
          console.log('404的错误~');
        } else if (err.response && err.response.status === 500) {
          // ElMessage({
          //   type: 'error',
          //   message: err.response.data.message
          // });
        }
        return Promise.reject(err);
      }
    );
  }

  request(config) {
    return new Promise((resolve, reject) => {
      // 1.单个请求对请求config的处理
      if (config.interceptors?.requestInterceptor) {
        config = config.interceptors.requestInterceptor(config);
      }

      // 2.判断是否需要显示loading
      if (config.showLoading === false) {
        this.showLoading = config.showLoading;
      }

      this.instance
        .request(config)
        .then((res) => {
          // 1.单个请求对数据的处理
          if (config.interceptors?.responseInterceptor) {
            res = config.interceptors.responseInterceptor(res);
          }
          // 2.将showLoading设置true, 这样不会影响下一个请求
          this.showLoading = DEAFULT_LOADING;

          // 3.将结果resolve返回出去
          resolve(res);
        })
        .catch((err) => {
          // 将showLoading设置true, 这样不会影响下一个请求
          this.showLoading = DEAFULT_LOADING;
          reject(err);
          return err;
        });
    });
  }

  get(config) {
    if (config.data && Object.keys(config.data)?.length) {
      config.url = `${config.url}?${qs.stringify(config.data)}`;
    }
    return this.request({ ...config, method: 'GET' });
  }

  post(config) {
    return this.request({ ...config, method: 'POST' });
  }

  delete(config) {
    return this.request({ ...config, method: 'DELETE' });
  }

  patch(config) {
    return this.request({ ...config, method: 'PATCH' });
  }
}

export default AiotRequest;
